@@ -26,9 +26,9 @@
 #include <stdio.h>
 #include <string.h>
 
-#include "sudo_gettext.h"	/* must be included before sudo_compat.h */
 #include "sudo_compat.h"
 #include "sudo_debug.h"
+#include "sudo_gettext.h"
 #include "sudo_util.h"
 
 /*
