@@ -1,7 +1,7 @@
 /*
  * SPDX-License-Identifier: ISC
  *
- * Copyright (c) 2019-2020 Todd C. Miller <Todd.Miller@sudo.ws>
+ * Copyright (c) 2019-2021 Todd C. Miller <Todd.Miller@sudo.ws>
  *
  * Permission to use, copy, modify, and distribute this software for any
  * purpose with or without fee is hereby granted, provided that the above
@@ -33,6 +33,7 @@
 #else
 # include "compat/stdbool.h"
 #endif
+#include <stddef.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -61,14 +62,29 @@
 # define DEFAULT_CA_CERT_PATH       "/etc/ssl/sudo/cacert.pem"
 # define DEFAULT_SERVER_CERT_PATH   "/etc/ssl/sudo/certs/logsrvd_cert.pem"
 # define DEFAULT_SERVER_KEY_PATH    "/etc/ssl/sudo/private/logsrvd_key.pem"
+
+/* Evaluates to true if at least one TLS field is set, else false. */
+# define TLS_CONFIGURED(_s)						\
+    ((_s).tls_key_path != NULL || (_s).tls_cert_path != NULL ||		\
+     (_s).tls_cacert_path != NULL || (_s).tls_dhparams_path != NULL ||	\
+     (_s).tls_ciphers_v12 != NULL || (_s).tls_ciphers_v13 != NULL ||	\
+     (_s).tls_verify != -1)
+
+/* Evaluates to the relay-specific TLS setting, falling back to server. */
+# define TLS_RELAY_STR(_c, _f)	\
+    ((_c)->relay._f != NULL ? (_c)->relay._f : (_c)->server._f)
+
+# define TLS_RELAY_INT(_c, _f)	\
+    ((_c)->relay._f != -1 ? (_c)->relay._f : (_c)->server._f)
 #endif
 
 struct logsrvd_config;
-typedef bool (*logsrvd_conf_cb_t)(struct logsrvd_config *config, const char *);
+typedef bool (*logsrvd_conf_cb_t)(struct logsrvd_config *, const char *, size_t);
 
 struct logsrvd_config_entry {
     char *conf_str;
     logsrvd_conf_cb_t setter;
+    size_t offset;
 };
 
 struct logsrvd_config_section {
@@ -76,17 +92,49 @@ struct logsrvd_config_section {
     struct logsrvd_config_entry *entries;
 };
 
+struct address_list_container {
+    unsigned int refcnt;
+    struct server_address_list addrs;
+};
+
 static struct logsrvd_config {
     struct logsrvd_config_server {
-        struct listen_address_list addresses;
+        struct address_list_container addresses;
         struct timespec timeout;
         bool tcp_keepalive;
 	char *pid_file;
 #if defined(HAVE_OPENSSL)
-        struct logsrvd_tls_config tls_config;
-        struct logsrvd_tls_runtime tls_runtime;
+	char *tls_key_path;
+	char *tls_cert_path;
+	char *tls_cacert_path;
+	char *tls_dhparams_path;
+	char *tls_ciphers_v12;
+	char *tls_ciphers_v13;
+	int tls_check_peer;
+	int tls_verify;
+	SSL_CTX *ssl_ctx;
 #endif
     } server;
+    struct logsrvd_config_relay {
+        struct address_list_container relays;
+        struct timespec connect_timeout;
+        struct timespec timeout;
+	time_t retry_interval;
+	char *relay_dir;
+        bool tcp_keepalive;
+	bool store_first;
+#if defined(HAVE_OPENSSL)
+	char *tls_key_path;
+	char *tls_cert_path;
+	char *tls_cacert_path;
+	char *tls_dhparams_path;
+	char *tls_ciphers_v12;
+	char *tls_ciphers_v13;
+	int tls_check_peer;
+	int tls_verify;
+	SSL_CTX *ssl_ctx;
+#endif
+    } relay;
     struct logsrvd_config_iolog {
 	bool compress;
 	bool flush;
@@ -136,14 +184,14 @@ logsrvd_conf_iolog_file(void)
 }
 
 /* server getters */
-struct listen_address_list *
-logsrvd_conf_listen_address(void)
+struct server_address_list *
+logsrvd_conf_server_listen_address(void)
 {
-    return &logsrvd_config->server.addresses;
+    return &logsrvd_config->server.addresses.addrs;
 }
 
 bool
-logsrvd_conf_tcp_keepalive(void)
+logsrvd_conf_server_tcp_keepalive(void)
 {
     return logsrvd_config->server.tcp_keepalive;
 }
@@ -155,32 +203,101 @@ logsrvd_conf_pid_file(void)
 }
 
 struct timespec *
-logsrvd_conf_get_sock_timeout(void)
+logsrvd_conf_server_timeout(void)
 {
     if (sudo_timespecisset(&logsrvd_config->server.timeout)) {
-        return &(logsrvd_config->server.timeout);
+        return &logsrvd_config->server.timeout;
     }
 
     return NULL;
 }
 
 #if defined(HAVE_OPENSSL)
-const struct logsrvd_tls_config *
-logsrvd_get_tls_config(void)
+SSL_CTX *
+logsrvd_server_tls_ctx(void)
 {
-    return &logsrvd_config->server.tls_config;
+    return logsrvd_config->server.ssl_ctx;
 }
 
-struct logsrvd_tls_runtime *
-logsrvd_get_tls_runtime(void)
+bool
+logsrvd_conf_server_tls_check_peer(void)
 {
-    return &logsrvd_config->server.tls_runtime;
+    return logsrvd_config->server.tls_check_peer;
+}
+#endif
+
+/* relay getters */
+struct server_address_list *
+logsrvd_conf_relay_address(void)
+{
+    return &logsrvd_config->relay.relays.addrs;
+}
+
+const char *
+logsrvd_conf_relay_dir(void)
+{
+    return logsrvd_config->relay.relay_dir;
+}
+
+bool
+logsrvd_conf_relay_store_first(void)
+{
+    return logsrvd_config->relay.store_first;
+}
+
+bool
+logsrvd_conf_relay_tcp_keepalive(void)
+{
+    return logsrvd_config->relay.tcp_keepalive;
+}
+
+struct timespec *
+logsrvd_conf_relay_timeout(void)
+{
+    if (sudo_timespecisset(&logsrvd_config->relay.timeout)) {
+        return &logsrvd_config->relay.timeout;
+    }
+
+    return NULL;
+}
+
+struct timespec *
+logsrvd_conf_relay_connect_timeout(void)
+{
+    if (sudo_timespecisset(&logsrvd_config->relay.connect_timeout)) {
+        return &logsrvd_config->relay.connect_timeout;
+    }
+
+    return NULL;
+}
+
+time_t
+logsrvd_conf_relay_retry_interval(void)
+{
+    return logsrvd_config->relay.retry_interval;
+}
+
+#if defined(HAVE_OPENSSL)
+SSL_CTX *
+logsrvd_relay_tls_ctx(void)
+{
+    if (logsrvd_config->relay.ssl_ctx != NULL)
+	return logsrvd_config->relay.ssl_ctx;
+    return logsrvd_config->server.ssl_ctx;
+}
+
+bool
+logsrvd_conf_relay_tls_check_peer(void)
+{
+    if (logsrvd_config->relay.tls_check_peer != -1)
+	return logsrvd_config->relay.tls_check_peer;
+    return logsrvd_config->server.tls_check_peer;
 }
 #endif
 
 /* I/O log callbacks */
 static bool
-cb_iolog_dir(struct logsrvd_config *config, const char *path)
+cb_iolog_dir(struct logsrvd_config *config, const char *path, size_t offset)
 {
     debug_decl(cb_iolog_dir, SUDO_DEBUG_UTIL);
 
@@ -193,7 +310,7 @@ cb_iolog_dir(struct logsrvd_config *config, const char *path)
 }
 
 static bool
-cb_iolog_file(struct logsrvd_config *config, const char *path)
+cb_iolog_file(struct logsrvd_config *config, const char *path, size_t offset)
 {
     debug_decl(cb_iolog_file, SUDO_DEBUG_UTIL);
 
@@ -206,7 +323,7 @@ cb_iolog_file(struct logsrvd_config *config, const char *path)
 }
 
 static bool
-cb_iolog_compress(struct logsrvd_config *config, const char *str)
+cb_iolog_compress(struct logsrvd_config *config, const char *str, size_t offset)
 {
     int val;
     debug_decl(cb_iolog_compress, SUDO_DEBUG_UTIL);
@@ -219,7 +336,7 @@ cb_iolog_compress(struct logsrvd_config *config, const char *str)
 }
 
 static bool
-cb_iolog_flush(struct logsrvd_config *config, const char *str)
+cb_iolog_flush(struct logsrvd_config *config, const char *str, size_t offset)
 {
     int val;
     debug_decl(cb_iolog_flush, SUDO_DEBUG_UTIL);
@@ -232,7 +349,7 @@ cb_iolog_flush(struct logsrvd_config *config, const char *str)
 }
 
 static bool
-cb_iolog_user(struct logsrvd_config *config, const char *user)
+cb_iolog_user(struct logsrvd_config *config, const char *user, size_t offset)
 {
     struct passwd *pw;
     debug_decl(cb_iolog_user, SUDO_DEBUG_UTIL);
@@ -250,7 +367,7 @@ cb_iolog_user(struct logsrvd_config *config, const char *user)
 }
 
 static bool
-cb_iolog_group(struct logsrvd_config *config, const char *group)
+cb_iolog_group(struct logsrvd_config *config, const char *group, size_t offset)
 {
     struct group *gr;
     debug_decl(cb_iolog_group, SUDO_DEBUG_UTIL);
@@ -267,7 +384,7 @@ cb_iolog_group(struct logsrvd_config *config, const char *group)
 }
 
 static bool
-cb_iolog_mode(struct logsrvd_config *config, const char *str)
+cb_iolog_mode(struct logsrvd_config *config, const char *str, size_t offset)
 {
     const char *errstr;
     mode_t mode;
@@ -284,7 +401,7 @@ cb_iolog_mode(struct logsrvd_config *config, const char *str)
 }
 
 static bool
-cb_iolog_maxseq(struct logsrvd_config *config, const char *str)
+cb_iolog_maxseq(struct logsrvd_config *config, const char *str, size_t offset)
 {
     const char *errstr;
     unsigned int value;
@@ -306,13 +423,15 @@ cb_iolog_maxseq(struct logsrvd_config *config, const char *str)
 
 /* Server callbacks */
 static bool
-cb_listen_address(struct logsrvd_config *config, const char *str)
+append_address(struct server_address_list *addresses, const char *str,
+    bool allow_wildcard)
 {
     struct addrinfo hints, *res, *res0 = NULL;
+    char *sa_str = NULL, *sa_host = NULL;
     char *copy, *host, *port;
     bool tls, ret = false;
     int error;
-    debug_decl(cb_iolog_mode, SUDO_DEBUG_UTIL);
+    debug_decl(append_address, SUDO_DEBUG_UTIL);
 
     if ((copy = strdup(str)) == NULL) {
 	sudo_warn(NULL);
@@ -323,8 +442,11 @@ cb_listen_address(struct logsrvd_config *config, const char *str)
     if (!iolog_parse_host_port(copy, &host, &port, &tls, DEFAULT_PORT,
 	    DEFAULT_PORT_TLS))
 	goto done;
-    if (host[0] == '*' && host[1] == '\0')
+    if (host[0] == '*' && host[1] == '\0') {
+	if (!allow_wildcard)
+	    goto done;
 	host = NULL;
+    }
 
 #if !defined(HAVE_OPENSSL)
     if (tls) {
@@ -333,6 +455,16 @@ cb_listen_address(struct logsrvd_config *config, const char *str)
     }
 #endif
 
+    /* Only make a single copy of the string + host for all addresses. */
+    if ((sa_str = sudo_rcstr_dup(str)) == NULL)	{
+	sudo_warn(NULL);
+	goto done;
+    }
+    if (host != NULL && (sa_host = sudo_rcstr_dup(host)) == NULL) {
+	sudo_warn(NULL);
+	goto done;
+    }
+
     /* Resolve host (and port if it is a service). */
     memset(&hints, 0, sizeof(hints));
     hints.ai_family = AF_UNSPEC;
@@ -344,25 +476,25 @@ cb_listen_address(struct logsrvd_config *config, const char *str)
 	goto done;
     }
     for (res = res0; res != NULL; res = res->ai_next) {
-	struct listen_address *addr;
+	struct server_address *addr;
 
 	if ((addr = malloc(sizeof(*addr))) == NULL) {
 	    sudo_warn(NULL);
 	    goto done;
 	}
-	if ((addr->sa_str = strdup(str)) == NULL) {
-	    sudo_warn(NULL);
-	    free(addr);
-	    goto done;
-	}
+	addr->sa_str = sudo_rcstr_addref(sa_str);
+	addr->sa_host = sudo_rcstr_addref(sa_host);
+
 	memcpy(&addr->sa_un, res->ai_addr, res->ai_addrlen);
 	addr->sa_size = res->ai_addrlen;
 	addr->tls = tls;
-	TAILQ_INSERT_TAIL(&config->server.addresses, addr, entries);
+	TAILQ_INSERT_TAIL(addresses, addr, entries);
     }
 
     ret = true;
 done:
+    sudo_rcstr_delref(sa_str);
+    sudo_rcstr_delref(sa_host);
     if (res0 != NULL)
 	freeaddrinfo(res0);
     free(copy);
@@ -370,11 +502,17 @@ done:
 }
 
 static bool
-cb_timeout(struct logsrvd_config *config, const char *str)
+cb_server_listen_address(struct logsrvd_config *config, const char *str, size_t offset)
+{
+    return append_address(&config->server.addresses.addrs, str, true);
+}
+
+static bool
+cb_server_timeout(struct logsrvd_config *config, const char *str, size_t offset)
 {
     int timeout;
     const char* errstr;
-    debug_decl(cb_timeout, SUDO_DEBUG_UTIL);
+    debug_decl(cb_server_timeout, SUDO_DEBUG_UTIL);
 
     timeout = sudo_strtonum(str, 0, UINT_MAX, &errstr);
     if (errstr != NULL)
@@ -386,10 +524,10 @@ cb_timeout(struct logsrvd_config *config, const char *str)
 }
 
 static bool
-cb_keepalive(struct logsrvd_config *config, const char *str)
+cb_server_keepalive(struct logsrvd_config *config, const char *str, size_t offset)
 {
     int val;
-    debug_decl(cb_keepalive, SUDO_DEBUG_UTIL);
+    debug_decl(cb_server_keepalive, SUDO_DEBUG_UTIL);
 
     if ((val = sudo_strtobool(str)) == -1)
 	debug_return_bool(false);
@@ -399,10 +537,10 @@ cb_keepalive(struct logsrvd_config *config, const char *str)
 }
 
 static bool
-cb_pid_file(struct logsrvd_config *config, const char *str)
+cb_server_pid_file(struct logsrvd_config *config, const char *str, size_t offset)
 {
     char *copy = NULL;
-    debug_decl(cb_pid_file, SUDO_DEBUG_UTIL);
+    debug_decl(cb_server_pid_file, SUDO_DEBUG_UTIL);
 
     /* An empty value means to disable the pid file. */
     if (*str != '\0') {
@@ -424,12 +562,13 @@ cb_pid_file(struct logsrvd_config *config, const char *str)
 
 #if defined(HAVE_OPENSSL)
 static bool
-cb_tls_key(struct logsrvd_config *config, const char *path)
+cb_tls_key(struct logsrvd_config *config, const char *path, size_t offset)
 {
+    char **p = (char **)((char *)config + offset);
     debug_decl(cb_tls_key, SUDO_DEBUG_UTIL);
 
-    free(config->server.tls_config.pkey_path);
-    if ((config->server.tls_config.pkey_path = strdup(path)) == NULL) {
+    free(*p);
+    if ((*p = strdup(path)) == NULL) {
         sudo_warn(NULL);
         debug_return_bool(false);
     }
@@ -437,12 +576,13 @@ cb_tls_key(struct logsrvd_config *config, const char *path)
 }
 
 static bool
-cb_tls_cacert(struct logsrvd_config *config, const char *path)
+cb_tls_cacert(struct logsrvd_config *config, const char *path, size_t offset)
 {
+    char **p = (char **)((char *)config + offset);
     debug_decl(cb_tls_cacert, SUDO_DEBUG_UTIL);
 
-    free(config->server.tls_config.cacert_path);
-    if ((config->server.tls_config.cacert_path = strdup(path)) == NULL) {
+    free(*p);
+    if ((*p = strdup(path)) == NULL) {
         sudo_warn(NULL);
         debug_return_bool(false);
     }
@@ -450,12 +590,13 @@ cb_tls_cacert(struct logsrvd_config *config, const char *path)
 }
 
 static bool
-cb_tls_cert(struct logsrvd_config *config, const char *path)
+cb_tls_cert(struct logsrvd_config *config, const char *path, size_t offset)
 {
+    char **p = (char **)((char *)config + offset);
     debug_decl(cb_tls_cert, SUDO_DEBUG_UTIL);
 
-    free(config->server.tls_config.cert_path);
-    if ((config->server.tls_config.cert_path = strdup(path)) == NULL) {
+    free(*p);
+    if ((*p = strdup(path)) == NULL) {
         sudo_warn(NULL);
         debug_return_bool(false);
     }
@@ -463,12 +604,13 @@ cb_tls_cert(struct logsrvd_config *config, const char *path)
 }
 
 static bool
-cb_tls_dhparam(struct logsrvd_config *config, const char *path)
+cb_tls_dhparams(struct logsrvd_config *config, const char *path, size_t offset)
 {
-    debug_decl(cb_tls_dhparam, SUDO_DEBUG_UTIL);
+    char **p = (char **)((char *)config + offset);
+    debug_decl(cb_tls_dhparams, SUDO_DEBUG_UTIL);
 
-    free(config->server.tls_config.dhparams_path);
-    if ((config->server.tls_config.dhparams_path = strdup(path)) == NULL) {
+    free(*p);
+    if ((*p = strdup(path)) == NULL) {
         sudo_warn(NULL);
         debug_return_bool(false);
     }
@@ -476,12 +618,13 @@ cb_tls_dhparam(struct logsrvd_config *config, const char *path)
 }
 
 static bool
-cb_tls_ciphers12(struct logsrvd_config *config, const char *str)
+cb_tls_ciphers12(struct logsrvd_config *config, const char *str, size_t offset)
 {
+    char **p = (char **)((char *)config + offset);
     debug_decl(cb_tls_ciphers12, SUDO_DEBUG_UTIL);
 
-    free(config->server.tls_config.ciphers_v12);
-    if ((config->server.tls_config.ciphers_v12 = strdup(str)) == NULL) {
+    free(*p);
+    if ((*p = strdup(str)) == NULL) {
         sudo_warn(NULL);
         debug_return_bool(false);
     }
@@ -489,12 +632,13 @@ cb_tls_ciphers12(struct logsrvd_config *config, const char *str)
 }
 
 static bool
-cb_tls_ciphers13(struct logsrvd_config *config, const char *str)
+cb_tls_ciphers13(struct logsrvd_config *config, const char *str, size_t offset)
 {
+    char **p = (char **)((char *)config + offset);
     debug_decl(cb_tls_ciphers13, SUDO_DEBUG_UTIL);
 
-    free(config->server.tls_config.ciphers_v13);
-    if ((config->server.tls_config.ciphers_v13 = strdup(str)) == NULL) {
+    free(*p);
+    if ((*p = strdup(str)) == NULL) {
         sudo_warn(NULL);
         debug_return_bool(false);
     }
@@ -502,35 +646,119 @@ cb_tls_ciphers13(struct logsrvd_config *config, const char *str)
 }
 
 static bool
-cb_tls_verify(struct logsrvd_config *config, const char *str)
+cb_tls_verify(struct logsrvd_config *config, const char *str, size_t offset)
 {
+    int *p = (int *)((char *)config + offset);
     int val;
     debug_decl(cb_tls_verify, SUDO_DEBUG_UTIL);
 
     if ((val = sudo_strtobool(str)) == -1)
 	debug_return_bool(false);
 
-    config->server.tls_config.verify = val;
+    *p = val;
     debug_return_bool(true);
 }
 
 static bool
-cb_tls_checkpeer(struct logsrvd_config *config, const char *str)
+cb_tls_checkpeer(struct logsrvd_config *config, const char *str, size_t offset)
 {
+    int *p = (int *)((char *)config + offset);
     int val;
     debug_decl(cb_tls_checkpeer, SUDO_DEBUG_UTIL);
 
     if ((val = sudo_strtobool(str)) == -1)
 	debug_return_bool(false);
 
-    config->server.tls_config.check_peer = val;
+    *p = val;
     debug_return_bool(true);
 }
 #endif
 
+/* relay callbacks */
+static bool
+cb_relay_host(struct logsrvd_config *config, const char *str, size_t offset)
+{
+    return append_address(&config->relay.relays.addrs, str, false);
+}
+
+static bool
+cb_relay_timeout(struct logsrvd_config *config, const char *str, size_t offset)
+{
+    int timeout;
+    const char* errstr;
+    debug_decl(cb_relay_timeout, SUDO_DEBUG_UTIL);
+
+    timeout = sudo_strtonum(str, 0, UINT_MAX, &errstr);
+    if (errstr != NULL)
+	debug_return_bool(false);
+
+    config->server.timeout.tv_sec = timeout;
+
+    debug_return_bool(true);
+}
+
+static bool
+cb_relay_connect_timeout(struct logsrvd_config *config, const char *str, size_t offset)
+{
+    int timeout;
+    const char* errstr;
+    debug_decl(cb_relay_connect_timeout, SUDO_DEBUG_UTIL);
+
+    timeout = sudo_strtonum(str, 0, UINT_MAX, &errstr);
+    if (errstr != NULL)
+	debug_return_bool(false);
+
+    config->relay.connect_timeout.tv_sec = timeout;
+
+    debug_return_bool(true);
+}
+
+static bool
+cb_relay_dir(struct logsrvd_config *config, const char *str, size_t offset)
+{
+    char *copy = NULL;
+    debug_decl(cb_relay_dir, SUDO_DEBUG_UTIL);
+
+    if ((copy = strdup(str)) == NULL) {
+	sudo_warn(NULL);
+	debug_return_bool(false);
+    }
+
+    free(config->relay.relay_dir);
+    config->relay.relay_dir = copy;
+
+    debug_return_bool(true);
+}
+
+static bool
+cb_relay_store_first(struct logsrvd_config *config, const char *str, size_t offset)
+{
+    int val;
+    debug_decl(cb_relay_store_first, SUDO_DEBUG_UTIL);
+
+    if ((val = sudo_strtobool(str)) == -1)
+	debug_return_bool(false);
+
+    config->relay.store_first = val;
+    debug_return_bool(true);
+}
+
+static bool
+cb_relay_keepalive(struct logsrvd_config *config, const char *str, size_t offset)
+{
+    int val;
+    debug_decl(cb_relay_keepalive, SUDO_DEBUG_UTIL);
+
+    if ((val = sudo_strtobool(str)) == -1)
+	debug_return_bool(false);
+
+    config->relay.tcp_keepalive = val;
+    debug_return_bool(true);
+}
+
 /* eventlog callbacks */
 static bool
-cb_eventlog_type(struct logsrvd_config *config, const char *str)
+cb_eventlog_type(struct logsrvd_config *config, const char *str, size_t offset)
 {
     debug_decl(cb_eventlog_type, SUDO_DEBUG_UTIL);
 
@@ -547,7 +775,7 @@ cb_eventlog_type(struct logsrvd_config *config, const char *str)
 }
 
 static bool
-cb_eventlog_format(struct logsrvd_config *config, const char *str)
+cb_eventlog_format(struct logsrvd_config *config, const char *str, size_t offset)
 {
     debug_decl(cb_eventlog_format, SUDO_DEBUG_UTIL);
 
@@ -563,7 +791,7 @@ cb_eventlog_format(struct logsrvd_config *config, const char *str)
 
 /* syslog callbacks */
 static bool
-cb_syslog_maxlen(struct logsrvd_config *config, const char *str)
+cb_syslog_maxlen(struct logsrvd_config *config, const char *str, size_t offset)
 {
     unsigned int maxlen;
     const char *errstr;
@@ -579,7 +807,7 @@ cb_syslog_maxlen(struct logsrvd_config *config, const char *str)
 }
 
 static bool
-cb_syslog_facility(struct logsrvd_config *config, const char *str)
+cb_syslog_facility(struct logsrvd_config *config, const char *str, size_t offset)
 {
     int logfac;
     debug_decl(cb_syslog_facility, SUDO_DEBUG_UTIL);
@@ -596,7 +824,7 @@ cb_syslog_facility(struct logsrvd_config *config, const char *str)
 }
 
 static bool
-cb_syslog_acceptpri(struct logsrvd_config *config, const char *str)
+cb_syslog_acceptpri(struct logsrvd_config *config, const char *str, size_t offset)
 {
     int logpri;
     debug_decl(cb_syslog_acceptpri, SUDO_DEBUG_UTIL);
@@ -613,7 +841,7 @@ cb_syslog_acceptpri(struct logsrvd_config *config, const char *str)
 }
 
 static bool
-cb_syslog_rejectpri(struct logsrvd_config *config, const char *str)
+cb_syslog_rejectpri(struct logsrvd_config *config, const char *str, size_t offset)
 {
     int logpri;
     debug_decl(cb_syslog_rejectpri, SUDO_DEBUG_UTIL);
@@ -627,7 +855,7 @@ cb_syslog_rejectpri(struct logsrvd_config *config, const char *str)
 }
 
 static bool
-cb_syslog_alertpri(struct logsrvd_config *config, const char *str)
+cb_syslog_alertpri(struct logsrvd_config *config, const char *str, size_t offset)
 {
     int logpri;
     debug_decl(cb_syslog_alertpri, SUDO_DEBUG_UTIL);
@@ -645,7 +873,7 @@ cb_syslog_alertpri(struct logsrvd_config *config, const char *str)
 
 /* logfile callbacks */
 static bool
-cb_logfile_path(struct logsrvd_config *config, const char *str)
+cb_logfile_path(struct logsrvd_config *config, const char *str, size_t offset)
 {
     char *copy = NULL;
     debug_decl(cb_logfile_path, SUDO_DEBUG_UTIL);
@@ -667,7 +895,7 @@ cb_logfile_path(struct logsrvd_config *config, const char *str)
 }
 
 static bool
-cb_logfile_time_format(struct logsrvd_config *config, const char *str)
+cb_logfile_time_format(struct logsrvd_config *config, const char *str, size_t offset)
 {
     char *copy = NULL;
     debug_decl(cb_logfile_time_format, SUDO_DEBUG_UTIL);
@@ -683,20 +911,64 @@ cb_logfile_time_format(struct logsrvd_config *config, const char *str)
     debug_return_bool(true);
 }
 
+void
+address_list_addref(struct server_address_list *al)
+{
+    struct address_list_container *container =
+	__containerof(al, struct address_list_container, addrs);
+    container->refcnt++;
+}
+
+void
+address_list_delref(struct server_address_list *al)
+{
+    struct address_list_container *container =
+	__containerof(al, struct address_list_container, addrs);
+    if (--container->refcnt == 0) {
+	struct server_address *addr;
+	while ((addr = TAILQ_FIRST(al))) {
+	    TAILQ_REMOVE(al, addr, entries);
+	    sudo_rcstr_delref(addr->sa_str);
+	    sudo_rcstr_delref(addr->sa_host);
+	    free(addr);
+	}
+    }
+}
+
 static struct logsrvd_config_entry server_conf_entries[] = {
-    { "listen_address", cb_listen_address },
-    { "timeout", cb_timeout },
-    { "tcp_keepalive", cb_keepalive },
-    { "pid_file", cb_pid_file },
+    { "listen_address", cb_server_listen_address },
+    { "timeout", cb_server_timeout },
+    { "tcp_keepalive", cb_server_keepalive },
+    { "pid_file", cb_server_pid_file },
 #if defined(HAVE_OPENSSL)
-    { "tls_key", cb_tls_key },
-    { "tls_cacert", cb_tls_cacert },
-    { "tls_cert", cb_tls_cert },
-    { "tls_dhparams", cb_tls_dhparam },
-    { "tls_ciphers_v12", cb_tls_ciphers12 },
-    { "tls_ciphers_v13", cb_tls_ciphers13 },
-    { "tls_checkpeer", cb_tls_checkpeer },
-    { "tls_verify", cb_tls_verify },
+    { "tls_key", cb_tls_key, offsetof(struct logsrvd_config, server.tls_key_path) },
+    { "tls_cacert", cb_tls_cacert, offsetof(struct logsrvd_config, server.tls_cacert_path) },
+    { "tls_cert", cb_tls_cert, offsetof(struct logsrvd_config, server.tls_cert_path) },
+    { "tls_dhparams", cb_tls_dhparams, offsetof(struct logsrvd_config, server.tls_dhparams_path) },
+    { "tls_ciphers_v12", cb_tls_ciphers12, offsetof(struct logsrvd_config, server.tls_ciphers_v12) },
+    { "tls_ciphers_v13", cb_tls_ciphers13, offsetof(struct logsrvd_config, server.tls_ciphers_v13) },
+    { "tls_checkpeer", cb_tls_checkpeer, offsetof(struct logsrvd_config, server.tls_check_peer) },
+    { "tls_verify", cb_tls_verify, offsetof(struct logsrvd_config, server.tls_verify) },
+#endif
+    { NULL }
+};
+
+static struct logsrvd_config_entry relay_conf_entries[] = {
+    { "relay_host", cb_relay_host },
+    { "timeout", cb_relay_timeout },
+    { "connect_timeout", cb_relay_connect_timeout },
+    { "relay_dir", cb_relay_dir },
+    { "store_first", cb_relay_store_first },
+    { "tcp_keepalive", cb_relay_keepalive },
+#if defined(HAVE_OPENSSL)
+    { "tls_key", cb_tls_key, offsetof(struct logsrvd_config, relay.tls_key_path) },
+    { "tls_cacert", cb_tls_cacert, offsetof(struct logsrvd_config, relay.tls_cacert_path) },
+    { "tls_cert", cb_tls_cert, offsetof(struct logsrvd_config, relay.tls_cert_path) },
+    { "tls_dhparams", cb_tls_dhparams, offsetof(struct logsrvd_config, relay.tls_dhparams_path) },
+    { "tls_ciphers_v12", cb_tls_ciphers12, offsetof(struct logsrvd_config, relay.tls_ciphers_v12) },
+    { "tls_ciphers_v13", cb_tls_ciphers13, offsetof(struct logsrvd_config, relay.tls_ciphers_v13) },
+    { "tls_checkpeer", cb_tls_checkpeer, offsetof(struct logsrvd_config, relay.tls_check_peer) },
+    { "tls_verify", cb_tls_verify, offsetof(struct logsrvd_config, relay.tls_verify) },
 #endif
     { NULL }
 };
@@ -736,6 +1008,7 @@ static struct logsrvd_config_entry logfile_conf_entries[] = {
 
 static struct logsrvd_config_section logsrvd_config_sections[] = {
     { "server", server_conf_entries },
+    { "relay", relay_conf_entries },
     { "iolog", iolog_conf_entries },
     { "eventlog", eventlog_conf_entries },
     { "syslog", syslog_conf_entries },
@@ -804,7 +1077,7 @@ logsrvd_conf_parse(struct logsrvd_config *config, FILE *fp, const char *path)
 	*ep = '\0';
 	for (entry = conf_section->entries; entry->conf_str != NULL; entry++) {
 	    if (strcasecmp(line, entry->conf_str) == 0) {
-		if (!entry->setter(config, val)) {
+		if (!entry->setter(config, val, entry->offset)) {
 		    sudo_warnx(U_("invalid value for %s: %s"),
 			entry->conf_str, val);
 		    goto done;
@@ -888,22 +1161,43 @@ logsrvd_conf_eventlog_setconf(struct logsrvd_config *config)
 }
 
 /* Free the specified struct logsrvd_config and its contents. */
-void
+static void
 logsrvd_conf_free(struct logsrvd_config *config)
 {
-    struct listen_address *addr;
     debug_decl(logsrvd_conf_free, SUDO_DEBUG_UTIL);
 
     if (config == NULL)
 	debug_return;
 
     /* struct logsrvd_config_server */
-    while ((addr = TAILQ_FIRST(&config->server.addresses))) {
-	TAILQ_REMOVE(&config->server.addresses, addr, entries);
-	free(addr->sa_str);
-	free(addr);
-    }
+    address_list_delref(&config->server.addresses.addrs);
     free(config->server.pid_file);
+#if defined(HAVE_OPENSSL)
+    free(config->server.tls_key_path);
+    free(config->server.tls_cert_path);
+    free(config->server.tls_cacert_path);
+    free(config->server.tls_dhparams_path);
+    free(config->server.tls_ciphers_v12);
+    free(config->server.tls_ciphers_v13);
+
+    if (config->server.ssl_ctx != NULL)
+	SSL_CTX_free(config->server.ssl_ctx);
+#endif
+
+    /* struct logsrvd_config_relay */
+    address_list_delref(&config->relay.relays.addrs);
+    free(config->relay.relay_dir);
+#if defined(HAVE_OPENSSL)
+    free(config->relay.tls_key_path);
+    free(config->relay.tls_cert_path);
+    free(config->relay.tls_cacert_path);
+    free(config->relay.tls_dhparams_path);
+    free(config->relay.tls_ciphers_v12);
+    free(config->relay.tls_ciphers_v13);
+
+    if (config->relay.ssl_ctx != NULL)
+	SSL_CTX_free(config->relay.ssl_ctx);
+#endif
 
     /* struct logsrvd_config_iolog */
     free(config->iolog.iolog_dir);
@@ -915,25 +1209,13 @@ logsrvd_conf_free(struct logsrvd_config *config)
     if (config->logfile.stream != NULL)
 	fclose(config->logfile.stream);
 
-#if defined(HAVE_OPENSSL)
-    free(config->server.tls_config.pkey_path);
-    free(config->server.tls_config.cert_path);
-    free(config->server.tls_config.cacert_path);
-    free(config->server.tls_config.dhparams_path);
-    free(config->server.tls_config.ciphers_v12);
-    free(config->server.tls_config.ciphers_v13);
-
-    if (config->server.tls_runtime.ssl_ctx != NULL)
-	SSL_CTX_free(config->server.tls_runtime.ssl_ctx);
-#endif
-
     free(config);
 
     debug_return;
 }
 
 /* Allocate a new struct logsrvd_config and set default values. */
-struct logsrvd_config *
+static struct logsrvd_config *
 logsrvd_conf_alloc(void)
 {
     struct logsrvd_config *config;
@@ -944,8 +1226,23 @@ logsrvd_conf_alloc(void)
 	debug_return_ptr(NULL);
     }
 
+    /* Relay defaults */
+    TAILQ_INIT(&config->relay.relays.addrs);
+    config->relay.relays.refcnt = 1;
+    config->relay.timeout.tv_sec = DEFAULT_SOCKET_TIMEOUT_SEC;
+    config->relay.connect_timeout.tv_sec = DEFAULT_SOCKET_TIMEOUT_SEC;
+    config->relay.tcp_keepalive = true;
+    config->relay.retry_interval = 30;
+    if (!cb_relay_dir(config, _PATH_SUDO_RELAY_DIR, 0))
+	goto bad;
+#if defined(HAVE_OPENSSL)
+    config->relay.tls_verify = -1;
+    config->relay.tls_check_peer = -1;
+#endif
+
     /* Server defaults */
-    TAILQ_INIT(&config->server.addresses);
+    TAILQ_INIT(&config->server.addresses.addrs);
+    config->server.addresses.refcnt = 1;
     config->server.timeout.tv_sec = DEFAULT_SOCKET_TIMEOUT_SEC;
     config->server.tcp_keepalive = true;
     config->server.pid_file = strdup(_PATH_SUDO_LOGSRVD_PID);
@@ -960,26 +1257,26 @@ logsrvd_conf_alloc(void)
      * This ensures we don't enable TLS by default when it is not configured.
      */
     if (access(DEFAULT_CA_CERT_PATH, R_OK) == 0) {
-	config->server.tls_config.cacert_path = strdup(DEFAULT_CA_CERT_PATH);
-	if (config->server.tls_config.cacert_path == NULL) {
+	config->server.tls_cacert_path = strdup(DEFAULT_CA_CERT_PATH);
+	if (config->server.tls_cacert_path == NULL) {
 	    sudo_warn(NULL);
 	    goto bad;
 	}
     }
     if (access(DEFAULT_SERVER_CERT_PATH, R_OK) == 0) {
-	config->server.tls_config.cert_path = strdup(DEFAULT_SERVER_CERT_PATH);
-	if (config->server.tls_config.cert_path == NULL) {
+	config->server.tls_cert_path = strdup(DEFAULT_SERVER_CERT_PATH);
+	if (config->server.tls_cert_path == NULL) {
 	    sudo_warn(NULL);
 	    goto bad;
 	}
     }
-    config->server.tls_config.pkey_path = strdup(DEFAULT_SERVER_KEY_PATH);
-    if (config->server.tls_config.pkey_path == NULL) {
+    config->server.tls_key_path = strdup(DEFAULT_SERVER_KEY_PATH);
+    if (config->server.tls_key_path == NULL) {
 	sudo_warn(NULL);
 	goto bad;
     }
-    config->server.tls_config.verify = true;
-    config->server.tls_config.check_peer = false;
+    config->server.tls_verify = true;
+    config->server.tls_check_peer = false;
 #endif
 
     /* I/O log defaults */
@@ -987,9 +1284,9 @@ logsrvd_conf_alloc(void)
     config->iolog.flush = true;
     config->iolog.mode = S_IRUSR|S_IWUSR;
     config->iolog.maxseq = SESSID_MAX;
-    if (!cb_iolog_dir(config, _PATH_SUDO_IO_LOGDIR))
+    if (!cb_iolog_dir(config, _PATH_SUDO_IO_LOGDIR, 0))
 	goto bad;
-    if (!cb_iolog_file(config, "%{seq}"))
+    if (!cb_iolog_file(config, "%{seq}", 0))
 	goto bad;
     config->iolog.uid = ROOT_UID;
     config->iolog.gid = ROOT_GID;
@@ -1001,27 +1298,27 @@ logsrvd_conf_alloc(void)
 
     /* Syslog defaults */
     config->syslog.maxlen = 960;
-    if (!cb_syslog_facility(config, LOGFAC)) {
+    if (!cb_syslog_facility(config, LOGFAC, 0)) {
 	sudo_warnx(U_("unknown syslog facility %s"), LOGFAC);
 	goto bad;
     }
-    if (!cb_syslog_acceptpri(config, PRI_SUCCESS)) {
+    if (!cb_syslog_acceptpri(config, PRI_SUCCESS, 0)) {
 	sudo_warnx(U_("unknown syslog priority %s"), PRI_SUCCESS);
 	goto bad;
     }
-    if (!cb_syslog_rejectpri(config, PRI_FAILURE)) {
+    if (!cb_syslog_rejectpri(config, PRI_FAILURE, 0)) {
 	sudo_warnx(U_("unknown syslog priority %s"), PRI_FAILURE);
 	goto bad;
     }
-    if (!cb_syslog_alertpri(config, PRI_FAILURE)) {
+    if (!cb_syslog_alertpri(config, PRI_FAILURE, 0)) {
 	sudo_warnx(U_("unknown syslog priority %s"), PRI_FAILURE);
 	goto bad;
     }
 
     /* Log file defaults */
-    if (!cb_logfile_time_format(config, "%h %e %T"))
+    if (!cb_logfile_time_format(config, "%h %e %T", 0))
 	goto bad;
-    if (!cb_logfile_path(config, _PATH_SUDO_LOGFILE))
+    if (!cb_logfile_path(config, _PATH_SUDO_LOGFILE, 0))
 	goto bad;
 
     debug_return_ptr(config);
@@ -1030,46 +1327,90 @@ bad:
     debug_return_ptr(NULL);
 }
 
-bool
+static bool
 logsrvd_conf_apply(struct logsrvd_config *config)
 {
+#if defined(HAVE_OPENSSL)
+    struct server_address *addr;
+#endif
     debug_decl(logsrvd_conf_apply, SUDO_DEBUG_UTIL);
 
     /* There can be multiple addresses so we can't set a default earlier. */
-    if (TAILQ_EMPTY(&config->server.addresses)) {
+    if (TAILQ_EMPTY(&config->server.addresses.addrs)) {
 	/* Enable plaintext listender. */
-	if (!cb_listen_address(config, "*:" DEFAULT_PORT))
+	if (!cb_server_listen_address(config, "*:" DEFAULT_PORT, 0))
 	    debug_return_bool(false);
 #if defined(HAVE_OPENSSL)
 	/* If a certificate was specified, enable the TLS listener too. */
-	if (config->server.tls_config.cert_path != NULL) {
-	    if (!cb_listen_address(config, "*:" DEFAULT_PORT_TLS "(tls)"))
+	if (config->server.tls_cert_path != NULL) {
+	    if (!cb_server_listen_address(config, "*:" DEFAULT_PORT_TLS "(tls)", 0))
 		debug_return_bool(false);
 	}
     } else {
-	struct listen_address *addr;
-
 	/* Check that TLS configuration is valid. */
-	TAILQ_FOREACH(addr, &config->server.addresses, entries) {
+	TAILQ_FOREACH(addr, &config->server.addresses.addrs, entries) {
 	    if (!addr->tls)
 		continue;
 	    /*
 	     * If a TLS listener was explicitly enabled but the cert path
 	     * was not, use the default.
 	     */
-	    if (config->server.tls_config.cert_path == NULL) {
-		config->server.tls_config.cert_path =
+	    if (config->server.tls_cert_path == NULL) {
+		config->server.tls_cert_path =
 		    strdup(DEFAULT_SERVER_CERT_PATH);
-		if (config->server.tls_config.cert_path == NULL) {
+		if (config->server.tls_cert_path == NULL) {
 		    sudo_warn(NULL);
 		    debug_return_bool(false);
 		}
 	    }
 	    break;
 	}
-#endif
+#endif /* HAVE_OPENSSL */
     }
 
+#if defined(HAVE_OPENSSL)
+    TAILQ_FOREACH(addr, &config->server.addresses.addrs, entries) {
+	if (!addr->tls)
+	    continue;
+        /* Create a TLS context for the server. */
+	config->server.ssl_ctx = init_tls_context(
+	    config->server.tls_cacert_path, config->server.tls_cert_path,
+	    config->server.tls_key_path, config->server.tls_dhparams_path,
+	    config->server.tls_ciphers_v12, config->server.tls_ciphers_v13,
+	    config->server.tls_verify);
+	if (config->server.ssl_ctx == NULL) {
+	    sudo_warnx(U_("unable to initialize server TLS context"));
+	    debug_return_bool(false);
+	}
+	break;
+    }
+
+    if (TLS_CONFIGURED(config->relay)) {
+	TAILQ_FOREACH(addr, &config->relay.relays.addrs, entries) {
+	    if (!addr->tls)
+		continue;
+	    /* Create a TLS context for the relay. */
+	    config->relay.ssl_ctx = init_tls_context(
+		TLS_RELAY_STR(config, tls_cacert_path),
+		TLS_RELAY_STR(config, tls_cert_path),
+		TLS_RELAY_STR(config, tls_key_path),
+		TLS_RELAY_STR(config, tls_dhparams_path),
+		TLS_RELAY_STR(config, tls_ciphers_v12),
+		TLS_RELAY_STR(config, tls_ciphers_v13),
+		TLS_RELAY_INT(config, tls_verify));
+	    if (config->relay.ssl_ctx == NULL) {
+		sudo_warnx(U_("unable to initialize relay TLS context"));
+		debug_return_bool(false);
+	    }
+	    break;
+	}
+    }
+#endif /* HAVE_OPENSSL */
+
+    /* Clear store_first if not relaying. */
+    if (TAILQ_EMPTY(&config->relay.relays.addrs))
+	config->relay.store_first = false;
+
     /* Open event log if specified. */
     switch (config->eventlog.log_type) {
     case EVLOG_SYSLOG:
@@ -1141,3 +1482,14 @@ done:
 	fclose(fp);
     debug_return_bool(ret);
 }
+
+void
+logsrvd_conf_cleanup(void)
+{
+    debug_decl(logsrvd_conf_cleanup, SUDO_DEBUG_UTIL);
+
+    logsrvd_conf_free(logsrvd_config);
+    logsrvd_config = NULL;
+
+    debug_return;
+}
