@@ -1,7 +1,7 @@
 /*
  * SPDX-License-Identifier: ISC
  *
- * Copyright (c) 2019-2020 Todd C. Miller <Todd.Miller@sudo.ws>
+ * Copyright (c) 2019-2021 Todd C. Miller <Todd.Miller@sudo.ws>
  *
  * Permission to use, copy, modify, and distribute this software for any
  * purpose with or without fee is hereby granted, provided that the above
@@ -80,12 +80,7 @@
 TAILQ_HEAD(connection_list, client_closure);
 static struct connection_list connections = TAILQ_HEAD_INITIALIZER(connections);
 
-static const char *server_name = "localhost";
-#if defined(HAVE_STRUCT_IN6_ADDR)
-static char server_ip[INET6_ADDRSTRLEN];
-#else
-static char server_ip[INET_ADDRSTRLEN];
-#endif
+static struct peer_info server_info = { "localhost" };
 static char *iolog_dir;
 static bool testrun = false;
 static int nr_of_conns = 1;
@@ -112,7 +107,7 @@ usage(bool fatal)
 #else
     fprintf(stderr, "usage: %s [-AnV] [-h host] [-i iolog-id] [-p port] "
 #endif
-	"[-r restart-point] [-R reject-reason] [-t number] /path/to/iolog\n",
+	"[-r restart-point] [-R reject-reason] [-s stop-point] [-t number] /path/to/iolog\n",
         getprogname());
     if (fatal)
 	exit(EXIT_FAILURE);
@@ -151,6 +146,8 @@ help(void)
 	_("restart previous I/O log transfer"));
     printf("  -R, --reject          %s\n",
 	_("reject the command with the given reason"));
+    printf("  -s, --stop-after        %s\n",
+	_("stop transfer after reaching this time"));
     printf("  -t, --test            %s\n",
 	_("test audit server by sending selected I/O log n times in parallel"));
     printf("  -V, --version         %s\n",
@@ -165,7 +162,7 @@ help(void)
  * Returns open socket or -1 on error.
  */
 static int
-connect_server(const char *host, const char *port)
+connect_server(struct peer_info *server, const char *port)
 {
     struct addrinfo hints, *res, *res0;
     const char *addr, *cause = "getaddrinfo";
@@ -175,9 +172,9 @@ connect_server(const char *host, const char *port)
     memset(&hints, 0, sizeof(hints));
     hints.ai_family = AF_UNSPEC;
     hints.ai_socktype = SOCK_STREAM;
-    error = getaddrinfo(host, port, &hints, &res0);
+    error = getaddrinfo(server->name, port, &hints, &res0);
     if (error != 0) {
-	sudo_warnx(U_("unable to look up %s:%s: %s"), host, port,
+	sudo_warnx(U_("unable to look up %s:%s: %s"), server->name, port,
 	    gai_strerror(error));
 	debug_return_int(-1);
     }
@@ -197,7 +194,7 @@ connect_server(const char *host, const char *port)
 	    sock = -1;
 	    continue;
 	}
-	if (*server_ip == '\0') {
+	if (server->ipaddr[0] == '\0') {
 	    switch (res->ai_family) {
 	    case AF_INET:
 		addr = (char *)&((struct sockaddr_in *)res->ai_addr)->sin_addr;
@@ -213,8 +210,8 @@ connect_server(const char *host, const char *port)
 		sock = -1;
 		continue;
 	    }
-	    if (inet_ntop(res->ai_family, addr, server_ip,
-		    sizeof(server_ip)) == NULL) {
+	    if (inet_ntop(res->ai_family, addr, server->ipaddr,
+		    sizeof(server->ipaddr)) == NULL) {
 		sudo_warnx("%s", U_("unable to get server IP addr"));
 	    }
 	}
@@ -377,6 +374,15 @@ fmt_client_hello(struct client_closure *closure)
     debug_return_bool(ret);
 }
 
+#if defined(HAVE_OPENSSL)
+/* Wrapper for fmt_client_hello() called via tls_connect_cb() */
+static bool
+tls_start_fn(struct tls_client_closure *tls_client)
+{
+    return fmt_client_hello(tls_client->parent_closure);
+}
+#endif /* HAVE_OPENSSL */
+
 static void
 free_info_messages(InfoMessage **info_msgs, size_t n_info_msgs)
 {
@@ -843,7 +849,16 @@ again:
     }
 
     /* Track elapsed time for comparison with commit points. */
-    sudo_timespecadd(&timing->delay, &closure->elapsed, &closure->elapsed);
+    sudo_timespecadd(&closure->elapsed, &timing->delay, &closure->elapsed);
+
+    /* If there is a stopping point, make sure we haven't reached it. */
+    if (sudo_timespecisset(&closure->stop_after)) {
+	if (sudo_timespeccmp(&closure->elapsed, &closure->stop_after, >)) {
+	    /* Reached limit, force premature end. */
+	    sudo_timespecsub(&closure->elapsed, &timing->delay, &closure->elapsed);
+	    debug_return_bool(false);
+	}
+    }
 
     /* If we have a restart point, ignore records until we hit it. */
     if (sudo_timespecisset(&closure->restart)) {
@@ -1115,13 +1130,14 @@ server_msg_cb(int fd, int what, void *v)
 
 #if defined(HAVE_OPENSSL)
     if (cert != NULL) {
+	SSL *ssl = closure->tls_client.ssl;
 	sudo_debug_printf(SUDO_DEBUG_INFO, "%s: reading ServerMessage (TLS)", __func__);
-        nread = SSL_read(closure->ssl, buf->data + buf->len, buf->size - buf->len);
+        nread = SSL_read(ssl, buf->data + buf->len, buf->size - buf->len);
         if (nread <= 0) {
 	    const char *errstr;
 	    int err;
 
-            switch (SSL_get_error(closure->ssl, nread)) {
+            switch (SSL_get_error(ssl, nread)) {
 		case SSL_ERROR_ZERO_RETURN:
 		    /* ssl connection shutdown cleanly */
 		    nread = 0;
@@ -1260,11 +1276,12 @@ client_msg_cb(int fd, int what, void *v)
 
 #if defined(HAVE_OPENSSL)
     if (cert != NULL) {
-        nwritten = SSL_write(closure->ssl, buf->data + buf->off, buf->len - buf->off);
+	SSL *ssl = closure->tls_client.ssl;
+        nwritten = SSL_write(ssl, buf->data + buf->off, buf->len - buf->off);
         if (nwritten <= 0) {
 	    const char *errstr;
 
-            switch (SSL_get_error(closure->ssl, nwritten)) {
+            switch (SSL_get_error(ssl, nwritten)) {
 		case SSL_ERROR_ZERO_RETURN:
 		    /* ssl connection shutdown */
 		    goto bad;
@@ -1351,233 +1368,6 @@ parse_timespec(struct timespec *ts, char *strval)
     debug_return_bool(true);
 }
 
-#if defined(HAVE_OPENSSL)
-/*
- * Check that the server's certificate is valid that it contains the
- * server name or IP address.
- * Returns 0 if the cert is invalid, else 1.
- */
-static int
-verify_peer_identity(int preverify_ok, X509_STORE_CTX *ctx)
-{
-    X509 *current_cert;
-    X509 *peer_cert;
-    debug_decl(verify_peer_identity, SUDO_DEBUG_UTIL);
-
-    /* if pre-verification of the cert failed, just propagate that result back */
-    if (preverify_ok != 1) {
-        debug_return_int(0);
-    }
-
-    /* since this callback is called for each cert in the chain,
-     * check that current cert is the peer's certificate
-     */
-    current_cert = X509_STORE_CTX_get_current_cert(ctx);
-    peer_cert = X509_STORE_CTX_get0_cert(ctx);
-    if (current_cert != peer_cert) {
-        debug_return_int(1);
-    }
-
-    if (validate_hostname(peer_cert, server_name, server_ip, 0) == MatchFound) {
-        debug_return_int(1);
-    }
-
-    debug_return_int(0);
-}
-
-static SSL_CTX *
-init_tls_client_context(const char *ca_bundle_file, const char *cert_file, const char *key_file)
-{
-    const SSL_METHOD *method;
-    SSL_CTX *ctx = NULL;
-    debug_decl(init_tls_client_context, SUDO_DEBUG_UTIL);
-
-    SSL_library_init();
-    OpenSSL_add_all_algorithms();
-    SSL_load_error_strings();
-
-    if ((method = TLS_client_method()) == NULL) {
-        sudo_debug_printf(SUDO_DEBUG_ERROR|SUDO_DEBUG_LINENO,
-            "creation of SSL_METHOD failed: %s",
-            ERR_error_string(ERR_get_error(), NULL));
-        goto bad;
-    }
-    if ((ctx = SSL_CTX_new(method)) == NULL) {
-        sudo_debug_printf(SUDO_DEBUG_ERROR|SUDO_DEBUG_LINENO,
-            "creation of new SSL_CTX object failed: %s",
-            ERR_error_string(ERR_get_error(), NULL));
-        goto bad;
-    }
-#ifdef HAVE_SSL_CTX_SET_MIN_PROTO_VERSION
-    if (!SSL_CTX_set_min_proto_version(ctx, TLS1_2_VERSION)) {
-        sudo_debug_printf(SUDO_DEBUG_ERROR|SUDO_DEBUG_LINENO,
-            "unable to restrict min. protocol version: %s",
-            ERR_error_string(ERR_get_error(), NULL));
-        goto bad;
-    }
-#else
-    SSL_CTX_set_options(ctx,
-        SSL_OP_NO_SSLv2|SSL_OP_NO_SSLv3|SSL_OP_NO_TLSv1|SSL_OP_NO_TLSv1_1);
-#endif
-
-    if (cert_file) {
-        if (!SSL_CTX_use_certificate_chain_file(ctx, cert_file)) {
-            sudo_debug_printf(SUDO_DEBUG_ERROR|SUDO_DEBUG_LINENO,
-                "unable to load cert to the ssl context: %s",
-                ERR_error_string(ERR_get_error(), NULL));
-            goto bad;
-        }
-        if (!SSL_CTX_use_PrivateKey_file(ctx, key_file, X509_FILETYPE_PEM)) {
-            sudo_debug_printf(SUDO_DEBUG_ERROR|SUDO_DEBUG_LINENO,
-                "unable to load key to the ssl context: %s",
-                ERR_error_string(ERR_get_error(), NULL));
-            goto bad;
-        }
-    }
-
-    if (ca_bundle_file != NULL) {
-        /* sets the location of the CA bundle file for verification purposes */
-        if (SSL_CTX_load_verify_locations(ctx, ca_bundle_file, NULL) <= 0) {
-            sudo_debug_printf(SUDO_DEBUG_ERROR|SUDO_DEBUG_LINENO,
-                "calling SSL_CTX_load_verify_locations() failed: %s",
-                ERR_error_string(ERR_get_error(), NULL));
-            goto bad;
-        }
-    }
-
-    if (verify_server) {
-        /* verify server cert during the handshake */
-        SSL_CTX_set_verify(ctx, SSL_VERIFY_PEER, verify_peer_identity);
-    }
-
-    goto done;
-
-bad:
-    SSL_CTX_free(ctx);
-
-done:
-    debug_return_ptr(ctx);
-}
-
-static void
-tls_connect_cb(int sock, int what, void *v)
-{
-    struct client_closure *closure = v;
-    struct sudo_event_base *evbase = closure->evbase;
-    struct timespec timeo = { TLS_HANDSHAKE_TIMEO_SEC, 0 };
-    const char *errstr;
-    int con_stat;
-    debug_decl(tls_connect_cb, SUDO_DEBUG_UTIL);
-
-    if (what == SUDO_EV_TIMEOUT) {
-        sudo_warnx("%s", U_("TLS handshake timeout occurred"));
-        goto bad;
-    }
-
-    con_stat = SSL_connect(closure->ssl);
-
-    if (con_stat == 1) {
-	sudo_debug_printf(SUDO_DEBUG_INFO|SUDO_DEBUG_LINENO,
-	    "SSL_connect successful");
-        closure->tls_connect_state = true;
-    } else {
-        switch (SSL_get_error(closure->ssl, con_stat)) {
-            /* TLS handshake is not finished, reschedule event */
-            case SSL_ERROR_WANT_READ:
-		sudo_debug_printf(SUDO_DEBUG_NOTICE|SUDO_DEBUG_LINENO,
-		    "SSL_connect returns SSL_ERROR_WANT_READ");
-		if (what != SUDO_EV_READ) {
-		    if (sudo_ev_set(closure->tls_connect_ev, closure->sock,
-			    SUDO_EV_READ, tls_connect_cb, closure) == -1) {
-			sudo_warnx("%s", U_("unable to set event"));
-			goto bad;
-		    }
-		}
-                if (sudo_ev_add(evbase, closure->tls_connect_ev, &timeo, false) == -1) {
-                    sudo_warnx("%s", U_("unable to add event to queue"));
-		    goto bad;
-                }
-		break;
-            case SSL_ERROR_WANT_WRITE:
-		sudo_debug_printf(SUDO_DEBUG_NOTICE|SUDO_DEBUG_LINENO,
-		    "SSL_connect returns SSL_ERROR_WANT_WRITE");
-		if (what != SUDO_EV_WRITE) {
-		    if (sudo_ev_set(closure->tls_connect_ev, closure->sock,
-			    SUDO_EV_WRITE, tls_connect_cb, closure) == -1) {
-			sudo_warnx("%s", U_("unable to set event"));
-			goto bad;
-		    }
-		}
-                if (sudo_ev_add(evbase, closure->tls_connect_ev, &timeo, false) == -1) {
-                    sudo_warnx("%s", U_("unable to add event to queue"));
-		    goto bad;
-                }
-		break;
-	    case SSL_ERROR_SYSCALL:
-                sudo_warnx(U_("TLS connection failed: %s"), strerror(errno));
-		goto bad;
-            default:
-		errstr = ERR_reason_error_string(ERR_get_error());
-                sudo_warnx(U_("TLS connection failed: %s"), errstr);
-                goto bad;
-        }
-    }
-
-    if (closure->tls_connect_state) {
-	if (!testrun) {
-	    printf("Negotiated protocol version: %s\n", SSL_get_version(closure->ssl));
-	    printf("Negotiated ciphersuite: %s\n", SSL_get_cipher(closure->ssl));
-	}
-
-	/* Done with TLS connect, send ClientHello */
-	sudo_ev_free(closure->tls_connect_ev);
-	closure->tls_connect_ev = NULL;
-	if (!fmt_client_hello(closure))
-	    goto bad;
-    }
-
-    debug_return;
-
-bad:
-    sudo_ev_loopbreak(evbase);
-    debug_return;
-}
-
-static bool
-tls_setup(struct client_closure *closure)
-{
-    const char *errstr;
-    debug_decl(tls_setup, SUDO_DEBUG_UTIL);
-
-    if ((ssl_ctx = init_tls_client_context(ca_bundle, cert, key)) == NULL) {
-	errstr = ERR_reason_error_string(ERR_get_error());
-        sudo_warnx(U_("Unable to initialize ssl context: %s"), errstr);
-        goto bad;
-    }
-    if ((closure->ssl = SSL_new(ssl_ctx)) == NULL) {
-	errstr = ERR_reason_error_string(ERR_get_error());
-        sudo_warnx(U_("Unable to allocate ssl object: %s"), errstr);
-        goto bad;
-    }
-    if (SSL_set_fd(closure->ssl, closure->sock) <= 0) {
-	errstr = ERR_reason_error_string(ERR_get_error());
-        sudo_warnx(U_("Unable to attach socket to the ssl object: %s"),
-	    errstr);
-        goto bad;
-    }
-
-    if (sudo_ev_add(closure->evbase, closure->tls_connect_ev, NULL, false) == -1) {
-	sudo_warnx("%s", U_("unable to add event to queue"));
-	goto bad;
-    }
-
-    debug_return_bool(true);
-
-bad:
-    debug_return_bool(false);
-}
-#endif /* HAVE_OPENSSL */
-
 /*
  * Free client closure contents.
  */
@@ -1589,11 +1379,11 @@ client_closure_free(struct client_closure *closure)
     if (closure != NULL) {
 	TAILQ_REMOVE(&connections, closure, entries);
 #if defined(HAVE_OPENSSL)
-        if (closure->ssl != NULL) {
-            SSL_shutdown(closure->ssl);
-            SSL_free(closure->ssl);
+        if (closure->tls_client.ssl != NULL) {
+            SSL_shutdown(closure->tls_client.ssl);
+            SSL_free(closure->tls_client.ssl);
         }
-	sudo_ev_free(closure->tls_connect_ev);
+	sudo_ev_free(closure->tls_client.tls_connect_ev);
 #endif
         sudo_ev_free(closure->read_ev);
         sudo_ev_free(closure->write_ev);
@@ -1612,7 +1402,7 @@ client_closure_free(struct client_closure *closure)
  */
 static struct client_closure *
 client_closure_alloc(int sock, struct sudo_event_base *base,
-    struct timespec *elapsed, struct timespec *restart, const char *iolog_id,
+    struct timespec *restart, struct timespec *stop_after, const char *iolog_id,
     char *reject_reason, bool accept_only, struct eventlog *evlog)
 {
     struct client_closure *closure;
@@ -1631,10 +1421,10 @@ client_closure_alloc(int sock, struct sudo_event_base *base,
     closure->reject_reason = reject_reason;
     closure->evlog = evlog;
 
-    closure->elapsed.tv_sec = elapsed->tv_sec;
-    closure->elapsed.tv_nsec = elapsed->tv_nsec;
     closure->restart.tv_sec = restart->tv_sec;
     closure->restart.tv_nsec = restart->tv_nsec;
+    closure->stop_after.tv_sec = stop_after->tv_sec;
+    closure->stop_after.tv_nsec = stop_after->tv_nsec;
 
     closure->iolog_id = iolog_id;
 
@@ -1655,10 +1445,15 @@ client_closure_alloc(int sock, struct sudo_event_base *base,
 
 #if defined(HAVE_OPENSSL)
     if (cert != NULL) {
-	closure->tls_connect_ev = sudo_ev_alloc(sock, SUDO_EV_WRITE,
-	    tls_connect_cb, closure);
-	if (closure->tls_connect_ev == NULL)
+	closure->tls_client.tls_connect_ev = sudo_ev_alloc(sock, SUDO_EV_WRITE,
+	    tls_connect_cb, &closure->tls_client);
+	if (closure->tls_client.tls_connect_ev == NULL)
 	    goto bad;
+	closure->tls_client.evbase = base;
+	closure->tls_client.parent_closure = closure;
+	closure->tls_client.peer_name = &server_info;
+	closure->tls_client.connect_timeout.tv_sec = TLS_HANDSHAKE_TIMEO_SEC;
+	closure->tls_client.start_fn = tls_start_fn;
     }
 #endif
 
@@ -1669,9 +1464,9 @@ bad:
 }
 
 #if defined(HAVE_OPENSSL)
-static const char short_opts[] = "Ah:i:np:r:R:t:b:c:k:V";
+static const char short_opts[] = "Ah:i:np:r:R:s:t:b:c:k:V";
 #else
-static const char short_opts[] = "Ah:i:Ip:r:R:t:V";
+static const char short_opts[] = "Ah:i:Ip:r:R:t:s:V";
 #endif
 static struct option long_opts[] = {
     { "accept",		no_argument,		NULL,	'A' },
@@ -1681,6 +1476,7 @@ static struct option long_opts[] = {
     { "port",		required_argument,	NULL,	'p' },
     { "restart",	required_argument,	NULL,	'r' },
     { "reject",		required_argument,	NULL,	'R' },
+    { "stop-after",	required_argument,	NULL,	's' },
     { "test",	    	optional_argument,	NULL,	't' },
 #if defined(HAVE_OPENSSL)
     { "ca-bundle",	required_argument,	NULL,	'b' },
@@ -1702,7 +1498,7 @@ main(int argc, char *argv[])
     struct eventlog *evlog;
     const char *port = NULL;
     struct timespec restart = { 0, 0 };
-    struct timespec elapsed = { 0, 0 };
+    struct timespec stop_after = { 0, 0 };
     bool accept_only = false;
     char *reject_reason = NULL;
     const char *iolog_id = NULL;
@@ -1740,22 +1536,26 @@ main(int argc, char *argv[])
 	    accept_only = true;
 	    break;
 	case 'h':
-	    server_name = optarg;
+	    server_info.name = optarg;
 	    break;
 	case 'i':
 	    iolog_id = optarg;
 	    break;
-	case 'R':
-	    reject_reason = optarg;
-	    break;
 	case 'p':
 	    port = optarg;
 	    break;
+	case 'R':
+	    reject_reason = optarg;
+	    break;
 	case 'r':
 	    if (!parse_timespec(&restart, optarg))
 		goto bad;
 	    open_mode = "r+";
 	    break;
+	case 's':
+	    if (!parse_timespec(&stop_after, optarg))
+		goto bad;
+	    break;
 	case 't':
 	    nr_of_conns = sudo_strtonum(optarg, 1, INT_MAX, &errstr);
 	    if (errstr != NULL) {
@@ -1833,14 +1633,14 @@ main(int argc, char *argv[])
         printf("connecting clients...\n");
 
     for (int i = 0; i < nr_of_conns; i++) {
-        sock = connect_server(server_name, port);
+        sock = connect_server(&server_info, port);
         if (sock == -1)
             goto bad;
         
         if (!testrun)
-            printf("Connected to %s:%s\n", server_name, port);
+            printf("Connected to %s:%s\n", server_info.name, port);
 
-        closure = client_closure_alloc(sock, evbase, &elapsed, &restart,
+        closure = client_closure_alloc(sock, evbase, &restart, &stop_after,
 	    iolog_id, reject_reason, accept_only, evlog);
         if (closure == NULL)
             goto bad;
@@ -1856,7 +1656,8 @@ main(int argc, char *argv[])
 
 #if defined(HAVE_OPENSSL)
 	if (cert != NULL) {
-	    if (!tls_setup(closure))
+	    if (!tls_client_setup(closure->sock, ca_bundle, cert, key, NULL,
+		    NULL, NULL, verify_server, false, &closure->tls_client))
 		goto bad;
 	} else
 #endif
