@@ -457,12 +457,6 @@ bool sudoers_gc_remove(enum sudoers_gc_types type, void *ptr);
 void sudoers_gc_init(void);
 void sudoers_gc_run(void);
 
-/* rcstr.c */
-char *rcstr_dup(const char *src);
-char *rcstr_alloc(size_t len);
-char *rcstr_addref(const char *s);
-void rcstr_delref(const char *s);
-
 /* strlcpy_unesc.c */
 size_t strlcpy_unescape(char *dst, const char *src, size_t size);
 
