@@ -1,7 +1,7 @@
 /*
  * SPDX-License-Identifier: ISC
  *
- * Copyright (c) 2014-2020 Todd C. Miller <Todd.Miller@sudo.ws>
+ * Copyright (c) 2014-2021 Todd C. Miller <Todd.Miller@sudo.ws>
  *
  * Permission to use, copy, modify, and distribute this software for any
  * purpose with or without fee is hereby granted, provided that the above
@@ -304,6 +304,8 @@ ts_write(int fd, const char *fname, struct timestamp_entry *entry, off_t offset)
 
     if (offset == -1) {
 	old_eof = lseek(fd, 0, SEEK_CUR);
+	if (old_eof == -1)
+	    debug_return_ssize_t(-1);
 	nwritten = write(fd, entry, entry->size);
     } else {
 	old_eof = offset;
@@ -750,6 +752,9 @@ timestamp_close(void *vcookie)
     debug_return;
 }
 
+#define TIMESPEC_VALID(ts) \
+    ((ts)->tv_sec >= 0 && (ts)->tv_nsec >= 0 && (ts)->tv_nsec < 1000000000L)
+
 /*
  * Check the time stamp file and directory and return their status.
  * Called with the file position before the locked record to read.
@@ -801,13 +806,21 @@ timestamp_status(void *vcookie, struct passwd *pw)
 
     /* Make sure what we read matched the expected record. */
     if (entry.version != TS_VERSION || entry.size != nread) {
-	/* do something else? */
 	sudo_debug_printf(SUDO_DEBUG_ERROR|SUDO_DEBUG_LINENO,
 	    "invalid time stamp file @ %lld", (long long)cookie->pos);
 	status = TS_OLD;
 	goto done;
     }
 
+    /* Sanity check time stamps. */
+    if (!TIMESPEC_VALID(&entry.start_time) || !TIMESPEC_VALID(&entry.ts)) {
+	sudo_debug_printf(SUDO_DEBUG_ERROR|SUDO_DEBUG_LINENO,
+	    "invalid timespec in time stamp file @ %lld",
+	    (long long)cookie->pos);
+	status = TS_OLD;
+	goto done;
+    }
+
     if (ISSET(entry.flags, TS_DISABLED)) {
 	sudo_debug_printf(SUDO_DEBUG_DEBUG|SUDO_DEBUG_LINENO,
 	    "time stamp record disabled");
